﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace LatLonToXYSample
{
    public partial class PlotALatitudeAndLongitudePointOnTheMap : UserControl
    {
        public PlotALatitudeAndLongitudePointOnTheMap()
        {
            InitializeComponent();
        }

        private void WpfMapLoaded(object sender, RoutedEventArgs e)
        {
           InitializeMap();
           combox.SelectedIndex = 0;
        }

        private void InitializeMap()
        {
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;

            var layerOverlay = new LayerOverlay();
            wpfMap1.Overlays.Add(layerOverlay);

            var pointLayer = new InMemoryFeatureLayer();
            pointLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.PointType = PointType.Bitmap;
            pointLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.Image = new GeoImage(@"..\..\Data\United States.png");
            pointLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            var pointOverlay = new LayerOverlay();
            pointOverlay.TileType = TileType.SingleTile;
            pointOverlay.Layers.Add("PointLayer", pointLayer);
            wpfMap1.Overlays.Add("PointOverlay", pointOverlay);

            wpfMap1.CurrentExtent = new RectangleShape(-133.2515625, 89.2484375, 126.9046875, -88.290625);

            wpfMap1.Refresh();

        }

        private ShapeFileFeatureLayer GetShapeFileLayer()
        {
            var fLayer = new ShapeFileFeatureLayer(@"..\..\Data\oklahoma_highway.shp");
            fLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            var color = new GeoColor(0,0,0);
            var lineStyle = new LineStyle
                                {
                                    OuterPen = new GeoPen(color, 1)
                                };
            fLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = lineStyle;
            return fLayer;
        }
        
        private void ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ListBoxItem lbItem = combox.SelectedItem as ListBoxItem;
            if (lbItem != null) MapCoordinates = lbItem.Content.ToString();


            //wgs84 lat/lon and equivalent x/y for State Plane Oklahoma North NAD27, feet
            double latitude = 35.5948678;
            double longitude = -99.6787493; 
            double x = 1551377.44;
            double y = 172338.19;


            Feature feature;

            LayerOverlay pointOverlay = null;
            InMemoryFeatureLayer pointLayer = null;
            ManagedProj4Projection proj4Projection = new ManagedProj4Projection();

            if (MapCoordinates == "Lat/Lon")
            {
                feature = new Feature(longitude, latitude, "Point1");
                wpfMap1.MapUnit = GeographyUnit.DecimalDegree;
                proj4Projection.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
                proj4Projection.ExternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();

                pointOverlay = (LayerOverlay)wpfMap1.Overlays["PointOverlay"];
                pointLayer = (InMemoryFeatureLayer)pointOverlay.Layers["PointLayer"];
                pointLayer.FeatureSource.Projection = proj4Projection;

                ShapeFileFeatureLayer shapeLayer = GetShapeFileLayer();

                if (!pointOverlay.Layers.Contains("LatLonShapeLayer"))
                {
                    pointOverlay.Layers.Add("LatLonShapeLayer", shapeLayer);
                }

                wpfMap1.CurrentExtent = new RectangleShape(-99.7175, 35.61735, -99.63797, 35.57408);
            }
            else 
            {
                // MapCoordinates == "X/Y"
                feature = new Feature(x, y, "Point1");
                wpfMap1.MapUnit = GeographyUnit.Feet;
                proj4Projection.InternalProjectionParametersString = ManagedProj4Projection.GetEpsgParametersString(3501);
                proj4Projection.ExternalProjectionParametersString = ManagedProj4Projection.GetEpsgParametersString(3501);
                //proj4Projection.ExternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();

                pointOverlay = (LayerOverlay)wpfMap1.Overlays["PointOverlay"];
                pointLayer = (InMemoryFeatureLayer)pointOverlay.Layers["PointLayer"];
                pointLayer.FeatureSource.Projection = proj4Projection;

                ShapeFileFeatureLayer shapeLayer = GetShapeFileLayer();

                if (!pointOverlay.Layers.Contains("XYShapeLayer"))
                {
                    pointOverlay.Layers.Add("XYShapeLayer", shapeLayer);
                }

                var upperLeftPointShape = new PointShape(x - 100, y + 100);
                var lowerRightPointShape = new PointShape(x + 100, y - 100);
                wpfMap1.CurrentExtent = new RectangleShape(upperLeftPointShape, lowerRightPointShape);
       
            }
            
            pointLayer.InternalFeatures.Clear();
            if (!pointLayer.InternalFeatures.Contains("Point1"))
            {
                pointLayer.InternalFeatures.Add("Point1", feature);
            }

            pointOverlay.Refresh();
            wpfMap1.Refresh();

        }

        public string MapCoordinates { get; set; }
    }
}
